/*
 * Copyright (C) 2019 <KichikuouChrome@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
*/

#ifndef __TEXTHOOK__
#define __TEXTHOOK__

enum texthook_mode {
	TEXTHOOK_NONE,
	TEXTHOOK_PRINT,
	TEXTHOOK_COPY,
};

void texthook_set_mode(enum texthook_mode mode);
void texthook_set_suppression_list(const char *suppressions);
void texthook_message(const char *m);
void texthook_newline(void);
void texthook_nextpage(void);
void texthook_keywait(void);

#endif /* __TEXTHOOK__ */
