#ifndef __GRAPH_BLEND_AMAP_H__
#define __GRAPH_BLEND_AMAP_H__

#include "ags.h"

void gr_blend_alpha_map(surface_t *dst, int dx, int dy, surface_t *src, int sx, int sy, int sw, int sh);

#endif /* __GRAPH_BLEND_AMAP_H__ */
